# This Python file uses the following encoding: latin-1

# Defines all the functions and variables needed to control the motors with the WAGO controller
# July 2015, Nicolas Delerue
__author__ = 'delerue'


from bpm_constants import *


VERBOSE_DEFAULT=0

global write_to_motor_dated_log
write_to_motor_dated_log='/Users/delerue/Downloads/write_to_motor_log'

from writeclient import *
import time

# basic functions
def write_to_motor(msg, verbose=VERBOSE_DEFAULT):
    "This function sends a message to the motor controller"
    ret = writeclient(motor_controller_IP, motor_controller_port, msg, verbose,datedlog=write_to_motor_dated_log)
    time.sleep(0.5)
    return ret


def motor_send(message, verbose=VERBOSE_DEFAULT):
    "Writes a formatted message to a motor"
    if (len(message) <= 12):
        print 'Message length ', len(message), ' >', message, '<'
    while len(message) < 12:
        message += '0'
    if len(message) <= 12:
        return write_to_motor(message, verbose)
    else:
        if verbose == 1:
            print 'Command too long'
        return 'Command too long'


def motor_enable(motor, verbose=0):
    "Enables a motor"
    msg = str(motor) + 'CE'
    motor_send(msg, verbose)


def motor_disable(motor, verbose=0):
    "Enables a motor"
    msg = str(motor) + 'CD'
    motor_send(msg, verbose)


def motor_reset(motor, verbose=0):
    "Enables a motor"
    msg = str(motor) + 'CR'
    motor_send(msg, verbose)


def motor_start(motor, verbose=0):
    "Starts the movement"
    msg = str(motor) + 'GG'
    motor_send(msg, verbose)


def motor_stop(motor, verbose=0):
    "Stops the movement"
    msg = str(motor) + 'GS'
    motor_send(msg, verbose)

### Advaned functions
def motor_set(motor, param, value):
    "save parameter defined in the variable 'param', with value 'value'"
    motor=str(motor)
    if (param == 'velocity'):
        if (value > 65535):
            return 'value is too big'
        else:
            return motor_send(motor + ('SV%09d' % (abs(float(value)))))
    elif (param == 'acceleration'):
        if (value > 65535):
            return 'value is too big'
        else:
            return motor_send(motor + 'SA%09d' % (abs(value)))
    elif (param == 'deceleration'):
        if (value > 65535):
            return 'value is too big'
        else:
            return motor_send(motor + 'SD%09d'(abs(value)))
    elif (param == 'target_position'):
        if value > 0:
            direction = 'P'
        else:
            direction = 'p'
        return motor_send(motor + 'S' + direction + '%09d' % abs(value))
    elif (param == 'actual_position'):
        motor_enable(motor)
        motor_reset(motor)
        if value > 0:
            direction = 'C'
        else:
            direction = 'c'
        ret=motor_send(motor + 'S' + direction + '%09d' % abs(value))
        motor_disable(motor)
        return ret
    else:
        return 'Unknown parameter'


def motor_get(motor, param):
    """get parameter defined in variable param, from the motor driver"""
    motor=str(motor)
    if (param == 'velocity'):
        velocity = motor_send(motor + 'QV')
        velo=float(velocity[3:12])
        print velo
        return velo
    elif (param == 'acceleration'):
        acceleration = motor_send(motor + 'QA')
        accel=float(acceleration[3:12])
        print accel
        return accel
    elif (param == 'deceleration'):
        deceleration = motor_send([motor + 'QD'])
        return deceleration[3:12]
    elif (param == 'actual_speed'):
        actual_speed = motor_send(motor + 'QC')
        isign = 1
        if actual_speed[2] == 'c':
            isign = -1
        return isign * actual_speed[3:12]
    elif (param == 'actual_position'):
        actual_position = motor_send(motor + 'QP')
        if actual_position[2] == 'p':
            isign = -1
        else:
            isign = 1
        print actual_position[2:13]
        print actual_position[3:12]
        pos = isign * float(actual_position[3:12])
        print 'actual position=', pos
        return pos
    elif (param == 'status'):
        status = motor_send(motor + 'QS')
        status = status[3]
        if status == 'R':
            ret='READY'
        elif status == 'B':
            ret='BUSY'
        elif status == 'E':
            ret='ERROR'
        else:
            ret="Unknown "+status
        print ret
        return ret
    elif (param == 'reference'):
        reference = motor_send(motor + 'QR')
        if float(reference[3:12]) == 2:
            return 1
        else:
            return 0
    elif param == 'limit':
        limit = motor_send(motor + 'L')
        if (limit[1] == 'l'):
            return 'No limit'
        elif ((limit[1] == 'L') and (limit[2] == 'B')):
            return 'Back limit'
        elif ((limit[1] == 'L') and (limit[2] == 'F')):
            return "Front limit"
        else:
            return 'limit unknown'
    else:
        return 'Request unknown'


def motor_move_relative(motor, speed, steps):
    "Moves a motor with the given parameters"
    motor=str(motor)
    motor_set(motor, 'velocity', speed)
    motor_set(motor, 'target_position', steps)
    motor_send(motor + 'GR')
    motor_start(motor)

def move_motor_relative(motor, speed, steps):
    motor_move_relative(motor, speed, steps)

def motor_move_absolute(motor, speed, steps):
    "Moves a motor with the given parameters"
    motor=str(motor)
    motor_set(motor, 'velocity', speed)
    motor_set(motor, 'target_position', steps)
    motor_send(motor + 'GA')
    motor_start(motor)

def move_motor_absolute(motor,speed,steps):
    "Moves a motor with the given parameters"
    motor_move_absolute(motor, speed, steps)


def motor_wait_for_ready(motor,verbose=VERBOSE_DEFAULT):
    motor_status=motor_get(motor,'status')
    iloop=1
    while ((iloop<1000) and (motor_status!='READY')):
        time.sleep(1)
        iloop=iloop+1
        motor_status=motor_get(motor,'status')
        if (iloop%10==0):
            motor_pos=motor_status=motor_get(motor,'actual_position')
    if (motor_status=='READY'):
        return 1
    else:
        return 0

def controller_hello():
    "Sends a_va query to the motor and display reply"
    #    write_to_motor("a_va?",1)
    ret=write_to_motor("ca_va?", 1)
    print "controller returned: ",ret
    return ret

controller_hello()
move_motor_relative(1,1000,-1000)
time.sleep(1)
move_motor_relative(1,1000,1000)
